"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Route = Route;
exports.Redirect = Redirect;
exports.Fallback = Fallback;
exports.Scope = Scope;
exports.Router = Router;
exports.createMiddleware = exports.locationToRoute = exports.routeToLocation = exports.exited = exports.entered = exports.changedTo = exports.paramsReducer = exports.match = exports.goForward = exports.goBack = exports.go = exports.open = exports.replace = exports.push = exports.sync = exports.LOCATION_NOT_MATCHED = exports.ROUTE_NOT_MATCHED = exports.ROUTE_CHANGED = exports.LocationMatchError = exports.RouteMatchError = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _typeof2 = _interopRequireDefault(require("@babel/runtime/helpers/typeof"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _possibleConstructorReturn2 = _interopRequireDefault(require("@babel/runtime/helpers/possibleConstructorReturn"));

var _getPrototypeOf2 = _interopRequireDefault(require("@babel/runtime/helpers/getPrototypeOf"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _wrapNativeSuper2 = _interopRequireDefault(require("@babel/runtime/helpers/wrapNativeSuper"));

var _pathToRegexp = _interopRequireDefault(require("path-to-regexp"));

var _queryString = _interopRequireDefault(require("query-string"));

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

// Errors
var RouteMatchError =
/*#__PURE__*/
function (_Error) {
  (0, _inherits2["default"])(RouteMatchError, _Error);

  function RouteMatchError() {
    (0, _classCallCheck2["default"])(this, RouteMatchError);
    return (0, _possibleConstructorReturn2["default"])(this, (0, _getPrototypeOf2["default"])(RouteMatchError).apply(this, arguments));
  }

  return RouteMatchError;
}((0, _wrapNativeSuper2["default"])(Error));

exports.RouteMatchError = RouteMatchError;

var LocationMatchError =
/*#__PURE__*/
function (_Error2) {
  (0, _inherits2["default"])(LocationMatchError, _Error2);

  function LocationMatchError() {
    (0, _classCallCheck2["default"])(this, LocationMatchError);
    return (0, _possibleConstructorReturn2["default"])(this, (0, _getPrototypeOf2["default"])(LocationMatchError).apply(this, arguments));
  }

  return LocationMatchError;
}((0, _wrapNativeSuper2["default"])(Error)); // Action Types


exports.LocationMatchError = LocationMatchError;

var createType = function createType(type) {
  return '@@reduxRoutable/' + type;
};

var SYNC = createType('SYNC');
var PUSH = createType('PUSH');
var REPLACE = createType('REPLACE');
var OPEN = createType('OPEN');
var GO = createType('GO');
var GO_BACK = createType('GO_BACK');
var GO_FORWARD = createType('GO_FORWARD');
var ROUTE_CHANGED = createType('ROUTE_CHANGED');
exports.ROUTE_CHANGED = ROUTE_CHANGED;
var ROUTE_NOT_MATCHED = createType('ROUTE_NOT_MATCHED');
exports.ROUTE_NOT_MATCHED = ROUTE_NOT_MATCHED;
var LOCATION_NOT_MATCHED = createType('LOCATION_NOT_MATCHED'); // Action Creators

exports.LOCATION_NOT_MATCHED = LOCATION_NOT_MATCHED;

var sync = function sync() {
  return {
    type: SYNC
  };
};

exports.sync = sync;

var push = function push(route) {
  var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var hash = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  return {
    type: PUSH,
    payload: {
      route: route,
      params: params,
      hash: hash
    }
  };
};

exports.push = push;

var replace = function replace(route) {
  var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var hash = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  return {
    type: REPLACE,
    payload: {
      route: route,
      params: params,
      hash: hash
    }
  };
};

exports.replace = replace;

var open = function open(route) {
  var params = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var hash = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '';
  return {
    type: OPEN,
    payload: {
      route: route,
      params: params,
      hash: hash
    }
  };
};

exports.open = open;

var go = function go(offset) {
  return {
    type: GO,
    payload: offset
  };
};

exports.go = go;

var goBack = function goBack() {
  return {
    type: GO_BACK
  };
};

exports.goBack = goBack;

var goForward = function goForward() {
  return {
    type: GO_FORWARD
  };
};

exports.goForward = goForward;

var routeChanged = function routeChanged(route, params, hash, previous) {
  return {
    type: ROUTE_CHANGED,
    payload: {
      route: route,
      params: params,
      hash: hash
    },
    meta: {
      previous: previous
    }
  };
};

var routeNotMatched = function routeNotMatched(error, route, params, hash) {
  return {
    type: ROUTE_NOT_MATCHED,
    error: true,
    payload: error,
    meta: {
      route: route,
      params: params,
      hash: hash
    }
  };
};

var locationNotMatched = function locationNotMatched(error, location) {
  return {
    type: LOCATION_NOT_MATCHED,
    error: true,
    payload: error,
    meta: {
      location: location
    }
  };
}; // Router Configuration


var createConfig = function createConfig(constructor, properties) {
  var instance = Object.create(constructor.prototype);
  return Object.assign(instance, properties);
};

function Route(name) {
  var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

  if (name === undefined) {
    throw new TypeError("'name' cannot be undefined");
  }

  if (typeof path !== 'string') {
    throw new TypeError("".concat(path, " is not a string"));
  }

  return createConfig(Route, {
    name: name,
    path: path,
    pattern: (0, _pathToRegexp["default"])(path)
  });
}

function Redirect(to) {
  var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

  if (to === undefined) {
    throw new TypeError("'to' cannot be undefined");
  }

  if (typeof path !== 'string') {
    throw new TypeError("".concat(path, " is not a string"));
  }

  return createConfig(Redirect, {
    to: to,
    path: path,
    pattern: (0, _pathToRegexp["default"])(path)
  });
}

function Fallback(name) {
  var path = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : '';

  if (name === undefined) {
    throw new TypeError("'name' cannot be undefined");
  }

  if (typeof path !== 'string') {
    throw new TypeError("".concat(path, " is not a string"));
  }

  var pattern = (0, _pathToRegexp["default"])(path, null, {
    end: false
  });
  return createConfig(Fallback, {
    name: name,
    path: '',
    pattern: pattern
  });
}

function Scope(base, router) {
  if (typeof base !== 'string') {
    throw new TypeError("".concat(base, " is not a string"));
  }

  if (!(router instanceof Router)) {
    throw new TypeError("".concat(router, " is not a Router"));
  }

  var scopedChildren = router.children.map(function (child) {
    switch (child.constructor) {
      case Fallback:
        return Fallback(child.name, base + child.path);

      case Redirect:
        return Redirect(child.to, base + child.path);

      case Route:
        return Route(child.name, base + child.path);
    }
  });
  return createConfig(Scope, {
    base: base,
    children: scopedChildren
  });
}

function Router(children) {
  if (!(children instanceof Array)) {
    throw new TypeError("".concat(children, " is not an Array"));
  }

  var resolvedChildren = children.reduce(function (children, child) {
    switch (child.constructor) {
      case Scope:
        return children.concat(child.children);

      case Fallback:
      case Redirect:
      case Route:
        return children.concat([child]);

      default:
        throw new TypeError("".concat(child, " is not a valid Router child"));
    }
  }, []);
  return createConfig(Router, {
    children: resolvedChildren
  });
} // Helpers


var match = function match(route, matchable) {
  if ((0, _typeof2["default"])(matchable) === 'object') {
    switch (matchable.constructor) {
      case Route:
      case Fallback:
        return match(route, matchable.name);

      case Router:
      case Scope:
        return match(route, matchable.children);

      case Array:
        return matchable.some(function (child) {
          return match(route, child);
        });

      default:
        return route === matchable;
    }
  } else {
    return route === matchable;
  }
};

exports.match = match;

var paramsReducer = function paramsReducer(matchable) {
  for (var _len = arguments.length, rest = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    rest[_key - 1] = arguments[_key];
  }

  var _ref = rest.length === 1 ? [null].concat(rest) : rest,
      _ref2 = (0, _slicedToArray2["default"])(_ref, 2),
      awayVal = _ref2[0],
      paramsSelector = _ref2[1];

  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : awayVal;

    var _ref3 = arguments.length > 1 ? arguments[1] : undefined,
        type = _ref3.type,
        payload = _ref3.payload;

    if (type === ROUTE_CHANGED) {
      if (match(payload.route, matchable)) {
        return paramsSelector(payload.params);
      } else {
        return awayVal;
      }
    } else {
      return state;
    }
  };
};

exports.paramsReducer = paramsReducer;

var changedTo = function changedTo(matchable) {
  return function (_ref4) {
    var type = _ref4.type,
        payload = _ref4.payload;
    return type === ROUTE_CHANGED && match(payload.route, matchable);
  };
};

exports.changedTo = changedTo;

var entered = function entered(matchable) {
  return function (_ref5) {
    var type = _ref5.type,
        payload = _ref5.payload,
        meta = _ref5.meta;
    return type === ROUTE_CHANGED && match(payload.route, matchable) && (meta.previous === undefined || !match(meta.previous.route, matchable));
  };
};

exports.entered = entered;

var exited = function exited(matchable) {
  return function (_ref6) {
    var type = _ref6.type,
        payload = _ref6.payload,
        meta = _ref6.meta;
    return type === ROUTE_CHANGED && !match(payload.route, matchable) && meta.previous !== undefined && match(meta.previous.route, matchable);
  };
}; // Utilities


exports.exited = exited;

var getPathParamNames = function getPathParamNames(path) {
  return _pathToRegexp["default"].parse(path).filter(function (token) {
    return token instanceof Object;
  }).map(function (token) {
    return token.name;
  });
};

var keyFilter = function keyFilter(object, condition) {
  return Object.entries(object).reduce(function (params, _ref7) {
    var _ref8 = (0, _slicedToArray2["default"])(_ref7, 2),
        key = _ref8[0],
        val = _ref8[1];

    if (condition(key)) params[key] = val;
    return params;
  }, {});
}; // Route/Location Translation


var routeToLocation = function routeToLocation(router, name, params, hash) {
  var route = router.children.find(function (child) {
    return child instanceof Route && child.name === name;
  });

  if (route === undefined) {
    throw new RouteMatchError("No route matching route name: ".concat(name));
  }

  var pathParamNames = getPathParamNames(route.path);
  var pathParams = keyFilter(params, function (key) {
    return pathParamNames.includes(key);
  });
  var queryParams = keyFilter(params, function (key) {
    return !pathParamNames.includes(key);
  });

  var search = _queryString["default"].stringify(queryParams);

  var pathname;

  try {
    pathname = _pathToRegexp["default"].compile(route.path)(pathParams);
  } catch (error) {
    throw new RouteMatchError(error.message);
  }

  return {
    pathname: pathname,
    search: search,
    hash: hash
  };
};

exports.routeToLocation = routeToLocation;

var locationToRoute = function locationToRoute(router, _ref9) {
  var pathname = _ref9.pathname,
      search = _ref9.search,
      hash = _ref9.hash;
  var route = router.children.find(function (child) {
    return child.pattern.test(pathname);
  });

  if (route === undefined) {
    throw new LocationMatchError("No route matching location path: ".concat(pathname));
  }

  var pathParamNames = getPathParamNames(route.path);
  var pathParamValues = route.pattern.exec(pathname).slice(1);
  var pathParams = pathParamNames.reduce(function (params, name, index) {
    var value = pathParamValues[index];
    if (value !== undefined) params[name] = value;
    return params;
  }, {});

  var queryParams = _queryString["default"].parse(search);

  var params = _objectSpread({}, pathParams, {}, queryParams);

  return {
    route: route,
    params: params,
    hash: hash
  };
}; // Middleware


exports.locationToRoute = locationToRoute;

var isAbsoluteAction = function isAbsoluteAction(_ref10) {
  var type = _ref10.type;
  return [PUSH, REPLACE, OPEN].includes(type);
};

var isRelativeAction = function isRelativeAction(_ref11) {
  var type = _ref11.type;
  return [GO, GO_BACK, GO_FORWARD].includes(type);
};

var createMiddleware = function createMiddleware(router, history) {
  return function (store) {
    var previous;

    var historyListener = function historyListener(location) {
      try {
        var _locationToRoute = locationToRoute(router, location),
            route = _locationToRoute.route,
            params = _locationToRoute.params,
            hash = _locationToRoute.hash;

        if (route instanceof Redirect) {
          history.replace(routeToLocation(router, route.to, params, hash));
        } else {
          store.dispatch(routeChanged(route.name, params, hash, previous));
        }
      } catch (error) {
        if (error instanceof LocationMatchError) {
          store.dispatch(locationNotMatched(error, location));
        } else {
          throw error;
        }
      }
    };

    history.listen(historyListener);
    return function (next) {
      return function (action) {
        if (action.type === SYNC) {
          historyListener(history.location);
        } else if (isAbsoluteAction(action)) {
          var _action$payload = action.payload,
              route = _action$payload.route,
              params = _action$payload.params,
              hash = _action$payload.hash;

          try {
            var location = routeToLocation(router, route, params, hash);

            switch (action.type) {
              case PUSH:
                history.push(location);
                break;

              case REPLACE:
                history.replace(location);
                break;

              case OPEN:
                window.open(history.createHref(location));
                break;
            }
          } catch (error) {
            if (error instanceof RouteMatchError) {
              store.dispatch(routeNotMatched(error, route, params, hash));
            } else {
              throw error;
            }
          }
        } else if (isRelativeAction(action)) {
          switch (action.type) {
            case GO:
              history.go(action.payload);
              break;

            case GO_BACK:
              history.goBack();
              break;

            case GO_FORWARD:
              history.goForward();
              break;
          }
        } else {
          if (action.type === ROUTE_CHANGED) {
            previous = action.payload;
          }

          return next(action);
        }
      };
    };
  };
};

exports.createMiddleware = createMiddleware;